@@ -8,6 +8,12 @@ before the final test summary time. Config file options ``filename``,
 ``sort`` and ``restrict`` can be used to change where profiling
 information is saved and how it is presented.
 
+Load this plugin by running nose2 with the
+`--plugin=nose2.plugins.prof` option and activate it
+with the `--profile` option,or put the corresponding
+entries (`plugin` and `always_on`) in the respective
+sections of the configuration file.
+
 """
 try:
     import hotshot
@@ -25,6 +31,7 @@ __unittest = True
 
 
 class Profiler(events.Plugin):
+
     """Profile the test run"""
 
     configSection = 'profiler'
@@ -39,7 +46,7 @@ class Profiler(events.Plugin):
 
     def register(self):
         """Don't register if hotshot is not found"""
-        if  hotshot is None:
+        if hotshot is None:
             log.error("Unable to profile: hotshot module not available")
             return
         super(Profiler, self).register()
@@ -54,6 +61,7 @@ class Profiler(events.Plugin):
         """Output profiling results"""
         # write prof output to stream
         class Stream:
+
             def write(self, *msg):
                 for m in msg:
                     event.stream.write(m)
