@@ -1,9 +1,9 @@
 import sys
-import inspect
 import logging
+import unittest
 
 from nose2 import util
-from nose2.compat import unittest
+from nose2 import events
 
 log = logging.getLogger(__name__)
 
@@ -16,12 +16,14 @@ __unittest = True
 
 class LayerSuite(unittest.BaseTestSuite):
 
-    def __init__(self, tests=(), layer=None):
+    def __init__(self, session, tests=(), layer=None):
         super(LayerSuite, self).__init__(tests)
         self.layer = layer
         self.wasSetup = False
+        self.session = session
 
     def run(self, result):
+        self.handle_previous_test_teardown(result)
         if not self._safeMethodCall(self.setUp, result):
             return
         try:
@@ -37,17 +39,37 @@ class LayerSuite(unittest.BaseTestSuite):
             if self.wasSetup:
                 self._safeMethodCall(self.tearDown, result)
 
+    def handle_previous_test_teardown(self, result):
+        prev = getattr(result, '_previousTestClass', None)
+        if prev is None:
+            return
+        layer_attr = getattr(prev, 'layer', None)
+        if isinstance(layer_attr, LayerSuite):
+            return
+        try:
+            suite_obj = unittest.suite.TestSuite()
+            suite_obj._tearDownPreviousClass(None, result)
+            suite_obj._handleModuleTearDown(result)
+        finally:
+            result._previousTestClass = None
+
     def setUp(self):
         if self.layer is None:
             return
 
         setup = self._getBoundClassmethod(self.layer, 'setUp')
+
+        event = events.StartLayerSetupEvent(self.layer)
+        self.session.hooks.startLayerSetup(event)
+
         if setup:
             setup()
         self.wasSetup = True
 
+        event = events.StopLayerSetupEvent(self.layer)
+        self.session.hooks.stopLayerSetup(event)
+
     def setUpTest(self, test):
-        # FIXME hook call
         if self.layer is None:
             return
         # skip suites, to ensure test setup only runs once around each test
@@ -62,26 +84,42 @@ class LayerSuite(unittest.BaseTestSuite):
             return
         if getattr(test, '_layer_wasSetUp', False):
             return
+
+        event = events.StartLayerSetupTestEvent(self.layer, test)
+        self.session.hooks.startLayerSetupTest(event)
+
         self._allLayers(test, 'testSetUp')
         test._layer_wasSetUp = True
 
+        event = events.StopLayerSetupTestEvent(self.layer, test)
+        self.session.hooks.stopLayerSetupTest(event)
+
     def tearDownTest(self, test):
-        # FIXME hook call
         if self.layer is None:
             return
         if not getattr(test, '_layer_wasSetUp', None):
             return
+
+        event = events.StartLayerTeardownTestEvent(self.layer, test)
+        self.session.hooks.startLayerTeardownTest(event)
+
         self._allLayers(test, 'testTearDown', reverse=True)
+
+        event = events.StopLayerTeardownTestEvent(self.layer, test)
+        self.session.hooks.stopLayerTeardownTest(event)
         delattr(test, '_layer_wasSetUp')
 
     def tearDown(self):
-        # FIXME hook call
         if self.layer is None:
             return
 
         teardown = self._getBoundClassmethod(self.layer, 'tearDown')
+        event = events.StartLayerTeardownEvent(self.layer)
+        self.session.hooks.startLayerTeardown(event)
         if teardown:
             teardown()
+        event = events.StopLayerTeardownEvent(self.layer)
+        self.session.hooks.stopLayerTeardown(event)
 
     def _safeMethodCall(self, method, result, *args):
         try:
@@ -108,16 +146,15 @@ class LayerSuite(unittest.BaseTestSuite):
     def _inLayer(self, layer, test, method):
         meth = self._getBoundClassmethod(layer, method)
         if meth:
-            args, _, _, _ = inspect.getargspec(meth)
-            if len(args) > 1:
+            if util.num_expected_args(meth) > 1:
                 meth(test)
             else:
                 meth()
 
     def _getBoundClassmethod(self, cls, method):
         """
-        Use instead of :func:`getattr` to get only classmethods explicitly defined
-        on ``cls`` (not methods inherited from ancestors)
+        Use instead of :func:`getattr` to get only classmethods explicitly
+        defined on ``cls`` (not methods inherited from ancestors)
         """
         descriptor = cls.__dict__.get(method, None)
         if descriptor:
